package dev.fyloz.colorrecipesexplorer.service

import dev.fyloz.colorrecipesexplorer.model.*
import dev.fyloz.colorrecipesexplorer.repository.CompanyRepository
import org.springframework.context.annotation.Lazy
import org.springframework.stereotype.Service

interface CompanyService :
    ExternalNamedModelService<Company, CompanySaveDto, CompanyUpdateDto, Company, CompanyRepository> {
    /** Checks if the given [company] is used by one or more recipes. */
    fun isLinkedToRecipes(company: Company): Boolean
}

@Service
class CompanyServiceImpl(
    companyRepository: CompanyRepository,
    @Lazy val recipeService: RecipeService
) :
    AbstractExternalNamedModelService<Company, CompanySaveDto, CompanyUpdateDto, Company, CompanyRepository>(
        companyRepository
    ),
    CompanyService {
    override fun idNotFoundException(id: Long) = companyIdNotFoundException(id)
    override fun idAlreadyExistsException(id: Long) = companyIdAlreadyExistsException(id)
    override fun nameNotFoundException(name: String) = companyNameNotFoundException(name)
    override fun nameAlreadyExistsException(name: String) = companyNameAlreadyExistsException(name)

    override fun Company.toOutput() = this

    override fun isLinkedToRecipes(company: Company): Boolean = recipeService.existsByCompany(company)

    override fun update(entity: CompanyUpdateDto): Company {
        // Lazy loaded to prevent checking the database when not necessary
        val persistedCompany by lazy { getById(entity.id) }

        return update(with(entity) {
            company(
                id = id,
                name = if (name != null && name.isNotBlank()) name else persistedCompany.name
            )
        })
    }

    override fun delete(entity: Company) {
        if (!repository.canBeDeleted(entity.id!!)) throw cannotDeleteCompany(entity)
        super.delete(entity)
    }
}
